
/* "Species" - a CoreWars evolver.  Copyright (C) 2003 'Varfar' 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later
 * version. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA. 
 */

#include "reproduction.hpp"
#include "species.hpp"
#include "rand.hpp"
#include "opcode_branch_lookup.hpp"
using namespace std;

namespace { // anon
	
	namespace BRANCH {
	
		// this structure is used when doing CROSSOVER_BRANCH
		struct GENE {
			int start,
				stop,
				index;
			int len() const { return stop - start; }
			bool continues, // might execution follow this gene? e.g. can't be followed immediately by a DAT
				skips, // might execution jump the next instruction (e.g. terminates on a skip)
				links, // does the last gene point somewhere else?
				executable, // e.g. non DAT
				dat_padding; // all clear 
			void copy(const CWarrior &src,CWarrior &dest,const int offset) {
				int i, ofs=offset;
				if(safety_checks) {
					if((offset+len()) > MAXLENGTH)
						PANIC(MISC,"bad gene copy dest",NULL);
					if((start < 0) || (stop > src.len))
						PANIC(MISC,"bad gene copy src",NULL);
				}
				for(i=start; i<stop; ++i) {
					dest.code[ofs++] = src.code[i];
				}
			}
			void dump(ostream &out,const CWarrior &warrior,const char prefix) const {
				int i;
				char linebuf[128];
				dumpSummary(out,prefix);
				for(i=start; i<stop; ++i) {
					dis1(linebuf,warrior.code[i],warrior.kingdom()->coresize());
					out << linebuf << endl;
				}
				if(links)
					out << "(links..)" << endl;
				if(continues)
					out << "(continues..)" << endl;
				if(skips)
					out << "(skips..)" << endl;
				out << endl;
			}
			void dumpSummary(ostream &out,const char prefix,const bool allflags = false) const {
				out << prefix << index << " (gene " << start << " -> " << stop-1;
				if(executable)
					cout << " executable";
				if(dat_padding)
					cout << " dat-padding";
				if(allflags) {
					if(links)
						out << " links";
					if(continues)
						out << " continues";
					if(skips)
						out << " skips";
				}
				cout << ')' << endl;
			}
			bool isNonContinousExecutableGene() const {
				return (executable && !continues && !skips);
			}
		} gMother[MAXLENGTH+2], gFather[MAXLENGTH+2], gRandom[MAXLENGTH+2];
		
		CFitness::NUM fitness_mother, fitness_father;
		
		bool areGenesViable(const GENE *genes,const int len) {
			int i;
			for(i=0; i<len; ++i) {
				if(genes[i].isNonContinousExecutableGene())
					return true;
			}
			return false;
		}
		
		bool areParentsViable(const int mlen,const int flen) {
			return (areGenesViable(gMother,mlen) && areGenesViable(gFather,flen));
		}
		
		struct GENE_PLAN {
			enum PARENT {
				MOTHER,
				FATHER,
				NONE
			} parent;
			bool dat_padding;
			static const int BAD_LINK = -1;
			int len,
				link,
				start;
			const GENE *gene;
			void clear() {
				parent = NONE;
				dat_padding = true;
				len = 0;
				link = BAD_LINK;
				start = 0;
				gene = 0;
			}
			const char prefix() const {
				if(dat_padding) {
					if(NONE != parent)
						PANIC(MISC,"bad PARENT in DAT_PADDING GENE_PLAN",NULL);
					return 'd';
				}	else {
					switch(parent) {
						case MOTHER:
							return 'M';
						case FATHER:
							return 'F';
						default:
							PANIC(MISC,"bad PARENT in GENE_PLAN",NULL);
					}
				}
				return '?'; // ho hum
			}
			void dump(ostream &out) const {
				if(dat_padding) {
					if(NONE != parent)
						PANIC(MISC,"bad PARENT in DAT_PADDING GENE_PLAN",NULL);
					out << "DAT_PADDING " << len << " long" << endl;
				}	else {
					gene->dumpSummary(out,prefix(),true);
				}
			}
			void id(ostream &out) {
				if(dat_padding) {
					out << 'd' << len << ' ';
				} else {
					out << prefix() << gene->index << ' ';
				}
			}
			void id(string &info) {
				if(dat_padding) {
					info += 'd';
					string_append(info,len);
				} else {
					info += prefix();
					string_append(info,gene->index);
				}
				info += '\t';
			}
		} gChild[MAXLENGTH+2];
		
		int plan_count;
		
		void dumpPlan(ostream &out) {
			int i;
			for(i=0; i<plan_count; ++i) {
				gChild[i].dump(out);
			}
		}
		
		void initPlan() {
			plan_count = 0;
		}
		
		bool geneSpace() {
			return (plan_count < MAXLENGTH-1);
		}

		int appendDATPadding(const int len) {
			int ret = 0;
			if(!geneSpace())
				return ret;
			gChild[plan_count].dat_padding = true;
			gChild[plan_count].len = len;
			gChild[plan_count].gene = 0;
			gChild[plan_count].link = GENE_PLAN::BAD_LINK;
			gChild[plan_count].parent = GENE_PLAN::NONE;
			gChild[plan_count].start = 0;
			ret = gChild[plan_count].len;
			plan_count++;
			return ret;
		}
		
		int appendGene(const GENE *gene,const GENE_PLAN::PARENT parent) {
			int ret = 0;
			if(!geneSpace())
				return ret;
			if(0 == gene)
				PANIC(MISC,"null gene",NULL);
			gChild[plan_count].dat_padding = false;
			gChild[plan_count].len = gene->len();
			gChild[plan_count].gene = gene;
			gChild[plan_count].link = GENE_PLAN::BAD_LINK;
			gChild[plan_count].parent = parent;
			gChild[plan_count].start = 0;
			ret = gChild[plan_count].len;
			plan_count++;
			return ret;
		}
				
		int randomNonContinuousExecutableGene(const GENE *genes,const int len) {
			const int n = CRand::irand(len);
			int i;
			if(genes[n].isNonContinousExecutableGene())
				return n;
			// find next
			for(i=n+1; i<len; ++i) {
				if(genes[i].isNonContinousExecutableGene())
					return i;
			}
			// find prev
			for(i=0; i<n; ++i) {
				if(genes[i].isNonContinousExecutableGene())
					return i;
			}
			// if here, then no non-continuous-executable-genes!
			for(i=0; i<len; ++i) {
				cout << i << '\t' << genes[i].isNonContinousExecutableGene() << endl;
			}
			PANIC(MISC,"no non-continuous executable genes!",NULL);
		}
		
		bool isPlanViable() {
			int i;
			for(i=0; i<plan_count; ++i) {
				if(0 != gChild[i].gene) { // not a DAT Padding
					if(gChild[i].gene->isNonContinousExecutableGene())
						return true;
				}
			}
			return (0 < plan_count);
		}
		
		namespace PERMS {
		
			GENE_PLAN gene_temp[MAXLENGTH+2];
			int gene_perm[MAXLENGTH+2];
			bool gene_used[MAXLENGTH+2];
			
			bool checkPlan(const int i) { // checks an individual element
				if(i == 0) return true; // nothing to do
				const int prev = i-1, prevprev = prev-1;
				// continuation? (skips always also continue btw)
				if(0 != gChild[gene_perm[prev]].gene) { // has a gene?
					if(gChild[gene_perm[prev]].gene->continues) {
						if(0 == gChild[gene_perm[i]].gene)
							return false;
						if(!gChild[gene_perm[i]].gene->executable)
							return false;
					}
				}
				// what if the gene before the previous one skipped?
				if(prevprev >= 0) {
					if(0 != gChild[gene_perm[prevprev]].gene) { // has a gene?
						if(gChild[gene_perm[prevprev]].gene->skips) {
							if(2 < gChild[gene_perm[prev]].len) { // we know this is executable btw
								if(0 == gChild[gene_perm[i]].gene)
									return false;
								if(!gChild[gene_perm[i]].gene->executable)
									return false;
							}
						}
					}
				}
				// if here, then everything is (potentially) ok still
				return true;
			}
				
			void perm(const int i,bool &done) {
				int j;
				if(i == plan_count) {
					// this is a full perm
					done = checkPlan(plan_count);
				} else if(checkPlan(i)) { // this weeds out a lot of bad combos from the start
					for(j=0; (j<plan_count) && !done; j++) {
						// only partially there
						if(!gene_used[j]) {
							gene_perm[i] =j;
							gene_used[j] = true;
							perm(i+1,done); // recurse
							gene_used[j] = false;
						}
					}
				}				
			}
			
		} // PERMS namespace
			
		bool findPerm() {
			using namespace PERMS;
			int i;
			bool done = false;
			// blank temps
			for(i=0; i<plan_count; i++)
				gene_used[i] = false;
			gChild[plan_count].clear(); // this sentinal is used for testing the full perm for running off the end 
			// start the search
			perm(0,done);
			// store result?
			if(done) {
				for(i=0; i<plan_count; i++)
					gene_temp[i] = gChild[gene_perm[i]];
				for(i=0; i<plan_count; i++)
					gChild[i] = gene_temp[gene_perm[i]];
			}
			// done
			return done;
		}
		
		void linkPlan() {
			int i, link;
			// cout << "linkPlan()" << endl;
			for(i=0; i<plan_count; ++i) {
				//cout << "linking " << i << endl;
				if(0 != gChild[i].gene) {
					if(gChild[i].gene->links) {
						gChild[i].link = GENE_PLAN::BAD_LINK;
						do {
							link = CRand::irand(plan_count);
							if(0 != gChild[link].gene) // something there?
								if(gChild[link].gene->executable)
									gChild[i].link = link;
						} while(GENE_PLAN::BAD_LINK == gChild[i].link);
					}
				}
			}
		}
		
		int geneSrc(const CReproduction::NUM dat_padding,const bool fitness_bias) { // 0 = mother, 1 = father, 2 = dat padding
			const CReproduction::NUM
				prob = CRand::frand(1.0F),
				padding =  (1.0F - dat_padding);
			CReproduction::NUM
				mother, father,
				sum, factor;
			if(fitness_bias) {
				sum = fitness_mother + fitness_father;
				factor = sum / padding;
				mother = fitness_mother * factor;
				father = mother + (fitness_father * factor);
			} else {
				mother = padding / 2;
				father = mother*2;
			}
			if(prob <= mother)
				return 0;
			if(prob <= father)
				return 1;
			return 2;
		}
		
		void swapGenes(const int x,const int y) {
			const GENE_PLAN temp = gChild[x];
			gChild[x] = gChild[y];
			gChild[y] = temp;
		}
		
		int generatePlan(const CReproduction::NUM dat_padding,const int mlen,const int flen,const int target_len,const bool fitness_bias,bool &aborted) {
			const int MAX_OUTER_TRIES = 10;
			int len, i, outer_tries=0, tries=0;
			aborted = false;
			do {
				// first generate a viable plan
				//cout << "target length is " << target_len << endl;
				do {
					initPlan();
					// first, insist that a gene from each parent is used
					len = appendGene(&gMother[randomNonContinuousExecutableGene(gMother,mlen)],GENE_PLAN::MOTHER);
					len += appendGene(&gFather[randomNonContinuousExecutableGene(gFather,flen)],GENE_PLAN::FATHER);
					// now generate n random genes
					while((len < target_len) && geneSpace()) {
						switch(geneSrc(dat_padding,fitness_bias)) {
							// mother
							case 0:
								i = CRand::irand(mlen);
								if(!gMother[i].dat_padding) // ok?
									len += appendGene(&gMother[i],GENE_PLAN::MOTHER);
								break;
							// father
							case 1:
								i = CRand::irand(flen);
								if(!gFather[i].dat_padding) // ok?
									len += appendGene(&gFather[i],GENE_PLAN::FATHER);
								break;
							// dat padding
							case 2:
								// DAT paddings are between 3 and 7 long
								// TODO: make this configurable or cleverer?
								i = CRand::irand(5); 
								len += appendDATPadding(3+i);
						}
						// safety checks
						if(len >= MAXLENGTH) {
							while(len >= MAXLENGTH) {
								plan_count--; // remove
								len -= gChild[plan_count].len;
							}
							break;
						}
					}
				} while(!isPlanViable());
				//dumpPlan(cout);
				//cerr << "plan took " << tries << " tries" << endl;
				// scramble plan
				for(i=0; i<plan_count; ++i) {
					swapGenes(i,CRand::irand(plan_count));
				}
				// now find a viable perm of this plan
				aborted = findPerm();
			} while(aborted && (++outer_tries < MAX_OUTER_TRIES));
			if(!aborted) {
				linkPlan();
				//cerr << "link took " << tries << " cuml tries" << endl;
				if(0 == plan_count)
					PANIC(MISC,"zero plan",NULL);
			}
			return tries;
		}
		
		int randomStartPos() {
			int ret;
			while(true) { // break is in loop
				ret = CRand::irand(plan_count);
				if(0 == gChild[ret].gene)
					continue;
				if(gChild[ret].gene->executable) {
					ret = gChild[ret].start;
					break;
				}
			}
			return ret;
		}

		#ifdef DEBUG_STEP			
		void debugStep(const CWarrior &warrior,const int i,const int gene,const char *state) {
			char linebuf[128];
			dis1(linebuf,warrior.code[i],warrior.kingdom()->coresize());
			cout << gene << '\t' << i << ' ' << state << '\t' << linebuf << endl; */
		}
		#else
		// do nothing
		#define debugStep(warrior,i,gene,state)
		#endif
	
		/* findGenes() populates a lookup array of genes from a warrior, using a very rudimentary system */
		int findGenes(const CWarrior &warrior,GENE *genes) {
			// vars
			int i=0, j=0,len=0, count=0;
			enum {
				INIT,
				EXEC,
				DATS,
				PADDING,
				DONE // this gene mind
			} state = INIT;
			bool executable, dat_padding, skips, continues, links, busy = true;
			// find all genes
			while(busy) { // for each character
				// finished processing warriors?
				if(i >= warrior.len) {
					state = DONE;
				}
				switch(state) {
					case INIT:
						debugStep(warrior,i,count,"init");
						executable = (DAT != warrior.code[i].opcode());
						dat_padding = warrior.code[i].is_clear();
						skips = false;
						continues = OPCODE_CONTINUES[warrior.code[i].opcode()];
						links = OPCODE_IS_BRANCH[warrior.code[i].opcode()];
						if(executable)
							state = EXEC;
						else if(dat_padding)
							state = PADDING;
						else
							state = DATS;
						//i++;
						len = 0;
						break;
					case EXEC:
						debugStep(warrior,i,count,"exec");
						if(DAT == warrior.code[i].opcode()) {
							state = DONE;
						} else {
							links = OPCODE_IS_BRANCH[warrior.code[i].opcode()];
							if(links) {
								state = DONE;
								continues = (skips | OPCODE_CONTINUES[warrior.code[i].opcode()]); 
							} else {
								len++;
							}
							skips = OPCODE_IS_SKIP[warrior.code[i].opcode()];
							i++;
						}
						break;
					case DATS:
						debugStep(warrior,i,count,"dats");
						if((DAT != warrior.code[i].opcode()) || warrior.code[i].is_clear()) {
							state = DONE;
						} else {
							i++;
							len++;
						}
						break;
					case PADDING:
						debugStep(warrior,i,count,"padding");
						if(!warrior.code[i].is_clear()) {
							//state = DONE;
							state = INIT; // completely ignore this padding; don't even bother to add it
						} else {
							i++;
							len++;
						}
						break;
					case DONE:
						debugStep(warrior,i,count,"done");
						// append gene?
						if(count > 0) {
							genes[count-1].stop = j;
						}
						// and set everything
						genes[count].start = j;
						genes[count].stop = i;
						genes[count].index = count;
						genes[count].executable = executable;
						genes[count].skips = skips;
						genes[count].continues = continues;
						genes[count].links = links;
						genes[count].dat_padding = dat_padding;
						// finally all over?
						busy = (i < warrior.len);
						// and carry on
						state = INIT;
						count++;
						j = i;
						break;
					default:
						PANIC(MISC,"unknown state finding genes",NULL);
				}
			}
			// done
			return count;
		}
		
		void dumpGenes(ostream &out,const char prefix,const CWarrior &warrior,GENE *genes,const int len) {
			int i=0;
			for(i=0; i<len; ++i)
				genes[i].dump(out,warrior,prefix);
		}
		
	} // BRANCH namespace
	
	const bool FITNESS_BIAS_DEFAULT = false; // a bit of backwards compatibility
	const bool ADJUST_ADDRESSING_DEFAULT = true;

	// this is a YACE style dummy warrior to be killed by an evolving candidate, just to check the evolving candidate has _some_ teeth
	warrior_t runtest_dummy;
	bool runtest_dummy_flag = false; // need to create the runtest_dummy?
		
/*	void randomInstruction(CWarrior &warrior,const int index) {
		warrior.code[index].in = OP(warrior.species()->freq()->get_random_opcode(warrior,index),CRand::irand(MODIFIER_LAST),CRand::irand(ADDRMODE_LAST),CRand::irand(ADDRMODE_LAST));
		warrior.code[index].a = warrior.operands()->rnd();
		warrior.code[index].b = warrior.operands()->rnd();
	} */
	
	/* this check will ensure that there is a SPL, JMP or MOV.I between start and DAT */
	bool loops(const CWarrior &warrior) {
		unsigned int i;
		for(i=warrior.start; i<warrior.len; i++) {
			switch(warrior.code[i].opcode()) {
				// oh no, dead!
				case DAT: 
					return false;
				// instructions that (might) loop
				case SPL:
				case DJN:
				case JMZ:
				case JMN:
				case JMP:
					// could check for bad indirects?
					// could discourage JMP-0?
					return true;
				// possible imp?
				case MOV:
					// check it is a .I
					if(mI == warrior.code[i].modifier())
						return true;
					// could check for bad indirects?
				// else continue to next instruction
				default:
					break;
			}
		}
		// if here, doesn't loop
		return false;
	}

	/* this check will run the warrior for a short time in a mars to see if it commits suicide */
	bool runtest(const CWarrior &warrior) {
		const unsigned int
			RUNTEST_DUMMY_LEN = 12,
			RUNTEST_DUMMY_POS = warrior.kingdom()->coresize() >> 1;
		unsigned int i;
		if(!runtest_dummy_flag) {
			// header
			runtest_dummy.start=1;
			runtest_dummy.len = RUNTEST_DUMMY_LEN;
			// a djnstream-resistant bit
			i=0;
			runtest_dummy.code[i].in = OP(JMP, mF, IMMEDIATE, DIRECT); // a jmp #n is a wimp
			runtest_dummy.code[i].a = 5 + i; // some colour
			runtest_dummy.code[i].b = 10 + i; // some colour
			// body
			for(i=1; i<(RUNTEST_DUMMY_LEN-1); i++) {
				runtest_dummy.code[i].in = OP(NOP, mF, DIRECT, DIRECT);
				runtest_dummy.code[i].a = 5 + i; // some colour
				runtest_dummy.code[i].b = 10 + i; // some colour
			}
			// loop back to beginning
			i = RUNTEST_DUMMY_LEN-1;
			runtest_dummy.code[i].in = OP(JMP, mF, DIRECT, DIRECT);
			runtest_dummy.code[i].a = warrior.kingdom()->coresize()-runtest_dummy.len+2; // loop
			runtest_dummy.code[i].b = 10 + i; // some colour
			// done
			//discore(runtest_dummy.code,0,runtest_dummy.len,warrior.kingdom()->coresize());/* and print to stdout. */
			runtest_dummy_flag = true;
		}
		sim_clear_core();
		sim_reset_pspaces();
		warrior.load(0);
		sim_load_warrior(RUNTEST_DUMMY_POS,&(runtest_dummy.code[0]),runtest_dummy.len);
		warrior.kingdom()->core_check();
		switch(sim(2,warrior.start,RUNTEST_DUMMY_POS+runtest_dummy.start,warrior.kingdom()->cycles(),0)) {
			case  0: // warrior won
				return true;
			case 1: // runtest_dummy won
			case 2: // tie
				return false;
			case -1:
			default:
				PANIC(MARS,"runtest","critical mars error");
		}
		return false; // hmm, dumb compiler
	}

} // anon namespace

/***** CReproduction class implementation *************/

const char
	*CReproduction::MUTATION_DESCS[CReproduction::MUTATION_LAST] = {
		"mutation_none","mutation_opcode","mutation_modifier","mutation_addrmode_a","mutation_addrmode_b","mutation_operand_a","mutation_operand_b"},
	*CReproduction::RANDOM_COMPONENT_DESCS[CReproduction::RANDOM_COMPONENT_LAST] = {
		"random_none","random_crossover","random_insert","random_replace","random_delete"},
	*CReproduction::CROSSOVER_DESCS[CReproduction::CROSSOVER_LAST] = {
		"random","branch"};

CReproduction::CReproduction() {
	// at first, mutation is 0 (100% of mutation is in _mutation[NONE])
	int i;
	_crossover = CROSSOVER_RANDOM;
	for(i=0; i<MUTATION_LAST; ++i)
		_mutation[(MUTATION)i] = 0.0F;
	for(i=0; i<RANDOM_COMPONENT_LAST; ++i)
		_crossover_random[(RANDOM_COMPONENT)i] = 0.0F;
	_mutation[MUT_NONE] = 1.0F;
	_mutation_sum = _mutation[MUT_NONE];
	_survive = 1.0F; // never
	_dat_padding = 0.0F; // never
	_fitness_bias = FITNESS_BIAS_DEFAULT;
	_adjust_addressing = ADJUST_ADDRESSING_DEFAULT;
}

void CReproduction::read_ini(INIFile &ini) {
	KeyValuePair *kvp;
	int i;
	// crossover
	kvp = ini.get("crossover"); if(0 == kvp) PANIC(MISC,"crossover expected",NULL);
	_crossover = crossover(kvp->getValueAsChar());
	// crossover-type-specific vars
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			_crossover_random_sum = 0.0F;
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				kvp = ini.get(random_component_desc((RANDOM_COMPONENT)i)); if(0 == kvp) PANIC(MISC,random_component_desc((RANDOM_COMPONENT)i)," expected");
				_crossover_random[(RANDOM_COMPONENT)i] = kvp->getValueAsFloat();
				if(0.0F > _crossover_random[(RANDOM_COMPONENT)i]) PANIC(MISC,random_component_desc((RANDOM_COMPONENT)i)," cannot be negative");
				_crossover_random_sum += _crossover_random[(RANDOM_COMPONENT)i];
			}
			if(0.0F >= _crossover_random_sum) PANIC(MISC,"sum of random crossover components cannot be negative nor zero",NULL);
			kvp = ini.get("adjust_addressing");
			if(0 != kvp) { // specified?
				_adjust_addressing = kvp->getValueAsBool(ADJUST_ADDRESSING_DEFAULT);
			}
			break;
		case CROSSOVER_BRANCH:
			kvp = ini.get("dat_padding"); 
			if(0 != kvp) {
				_dat_padding = kvp->getValueAsFloat();
				if((0.0F > _dat_padding) || (_dat_padding >= 1.0F))
					PANIC(MISC,"dat_padding must be between 0 and 1",NULL);
			}
			kvp = ini.get("crossover_fitness_bias");
			if(0 != kvp) 
				_fitness_bias = kvp->getValueAsBool(FITNESS_BIAS_DEFAULT);
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	// survive
	kvp = ini.get("survive"); if(0 == kvp) PANIC(MISC,"survive expected",NULL);
	_survive = kvp->getValueAsFloat();
	if((0.0F > _survive) || (_survive >= 1.0F))
		PANIC(MISC,"survive must be between 0 and 1",NULL);
	// mutation
	for(i=0; i<MUTATION_LAST; ++i) {
		kvp = ini.get(mutation_desc((MUTATION)i)); if(0 == kvp) PANIC(MISC,mutation_desc((MUTATION)i)," expected");
		_mutation_sum -= _mutation[(MUTATION)i];
		_mutation[(MUTATION)i] = kvp->getValueAsFloat();
		if(_mutation[(MUTATION)i] < 0.0F)
			PANIC(MISC,mutation_desc((MUTATION)i),"must be >= 0");
		_mutation_sum += _mutation[(MUTATION)i];
	}
	if(_mutation_sum <= 0.0F)
		PANIC(MISC,"sum mutation rate is invalid",NULL);
}

CReproduction *CReproduction::read_override_ini(INIFile &ini) /* loads values; if any values are
	specified, then a copy of this reproduction will be made; all unspecified values in the new
	class will be "inherited" from this one, and any specified values will be overriden.  If nothing
	is specified, then this will be returned */ {
	CReproduction *ret = this;
	KeyValuePair *kvp;
	int i;
	// crossover
	kvp = ini.get("crossover");
	if(0 != kvp) { // specified? override then
		if(!kvp->isValue(crossover_desc(_crossover))) { // different?
			ret = new CReproduction(*this); // create a copy to return
			ret->_crossover = crossover(kvp->getValueAsChar());
		}
	}
	// get crossover-type-specific vars
	switch(ret->_crossover) {
		case CROSSOVER_RANDOM:
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				kvp = ini.get(random_component_desc((RANDOM_COMPONENT)i));
				if(0 != kvp) { // specified?  override then
					if(this == ret) { // nothing previously overriden?
						ret = new CReproduction(*this); // create a copy to return
						if(_crossover != ret->_crossover) {
							ret->_crossover_random_sum = 0.0F;
							for(int j=0; j<RANDOM_COMPONENT_LAST; j++)
								ret->_crossover_random[(RANDOM_COMPONENT)j] = 0.0F;
						}
					}
					ret->_crossover_random_sum -= ret->_crossover_random[(RANDOM_COMPONENT)i];
					ret->_crossover_random[(RANDOM_COMPONENT)i] = kvp->getValueAsFloat();
					ret->_crossover_random_sum += ret->_crossover_random[(RANDOM_COMPONENT)i];
					if(0.0F > _crossover_random[(RANDOM_COMPONENT)i])
						PANIC(MISC,random_component_desc((RANDOM_COMPONENT)i)," cannot be negative");
				}
			}
			kvp = ini.get("adjust_addressing");
			if(0 != kvp) { // set?
				if(kvp->getValueAsBool(ret->_adjust_addressing) != ret->_adjust_addressing) { // changed?
					if(this == ret) { // nothing previously overriden?
						ret = new CReproduction(*this); // create a copy to return
					}
					ret->_adjust_addressing = !ret->_adjust_addressing; // and toggle it
				}
			}
			break;
		case CROSSOVER_BRANCH:
			kvp = ini.get("dat_padding");
			if(0 != kvp) { // specified? override then
				if(this == ret) // nothing previously overriden?
					ret = new CReproduction(*this); // create a copy to return
				ret->_dat_padding = kvp->getValueAsFloat();
				if((0.0F > _dat_padding) || (_dat_padding >= 1.0F))
					PANIC(MISC,"dat_padding must be between 0 and 1",NULL);
			} else if(_crossover != ret->_crossover) { // else type is different
				PANIC(MISC,"dat_padding is expected",NULL);
			}
			kvp = ini.get("crossover_fitness_bias");
			if(0 != kvp) { // specified? override then
				if(this == ret) // nothing previously overriden?
					ret = new CReproduction(*this); // create a copy to return
				ret->_fitness_bias = kvp->getValueAsBool(FITNESS_BIAS_DEFAULT);
			}
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	// survive
	kvp = ini.get("survive");
	if(0 != kvp) { // specified?  override then
		if(this == ret) // crossover not already overriden?
			ret = new CReproduction(*this); // create a copy to return
		ret->_survive = kvp->getValueAsFloat();
		if((0.0F > _survive) || (_survive >= 1.0F))
			PANIC(MISC,"survive must be between 0 and 1",NULL);
	}
	// mutation
	for(i=0; i<MUTATION_LAST; ++i) {
		kvp = ini.get(mutation_desc((MUTATION)i));
		if(0 != kvp) { // specified?
			if(this == ret) { // crossover wasn't override?
				ret = new CReproduction(*this); // create a copy to return
			}
			ret->_mutation_sum -= ret->_mutation[(MUTATION)i];
			ret->_mutation[(MUTATION)i] = kvp->getValueAsFloat();
			if(ret->_mutation[(MUTATION)i] < 0.0F)
				PANIC(MISC,mutation_desc((MUTATION)i),"must be >= 0");
			ret->_mutation_sum += ret->_mutation[(MUTATION)i];
		}
	}
	if(ret->_mutation_sum <= 0.0F)
		PANIC(MISC,"sum mutation rate is invalid",NULL);
	// done
	return ret;
}

void CReproduction::write_ini(ostream &os) {
	int i;
	os << "crossover=" << crossover_desc(_crossover) << endl;
	// crossover type specific vars
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				os << random_component_desc((RANDOM_COMPONENT)i) << "=" << _crossover_random[(RANDOM_COMPONENT)i] << endl;
			}
			if(ADJUST_ADDRESSING_DEFAULT != _adjust_addressing) {
				os << "adjust_addressing=" << _adjust_addressing << endl;
			}
			break;
		case CROSSOVER_BRANCH:
			os << "dat_padding=" << _dat_padding << endl;
			if(FITNESS_BIAS_DEFAULT != _fitness_bias) // not default?
				os << "crossover_fitness_bias=" << _fitness_bias << endl;
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	os << "survive=" << _survive << endl;
	for(i=0; i<MUTATION_LAST; ++i) {
		os << mutation_desc((MUTATION)i) << '=' << _mutation[(MUTATION)i] << endl;
	}
}

void CReproduction::write_override_ini(ostream &os,const CReproduction *parent) {
	int i;
	if(this == parent) // not overriden?
		return;
	// crossover
	if(parent->_crossover != _crossover) {
		os << "crossover=" << crossover_desc(_crossover) << endl;
	}
	// crossover-type-specific vars
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			for(i=0; i<RANDOM_COMPONENT_LAST; i++) {
				if((parent->_crossover != _crossover) || (parent->_crossover_random[(RANDOM_COMPONENT)i] != _crossover_random[(RANDOM_COMPONENT)i])) {
					os << random_component_desc((RANDOM_COMPONENT)i) << "=" << _crossover_random[(RANDOM_COMPONENT)i] << endl;
				}
			}
			if(parent->_adjust_addressing != _adjust_addressing) {
				os << "adjust_addressing=" << _adjust_addressing << endl;
			}
			break;
		case CROSSOVER_BRANCH:
			if((parent->_crossover != _crossover) || (parent->_fitness_bias != _fitness_bias))
				os << "crossover_fitness_bias=" << _fitness_bias << endl;
			if((parent->_crossover != _crossover) || (parent->_dat_padding != _dat_padding)) {
				os << "dat_padding=" << _dat_padding << endl;
			}
			break;
		default:
			PANIC(MISC,"bad crossover type",NULL);
	}
	// survive
	if(parent->_survive != _survive) {
		os << "survive=" << _survive << endl;
	}
	// mutation
	for(i=0; i<MUTATION_LAST; ++i) {
		if(parent->_mutation[(MUTATION)i] != _mutation[(MUTATION)i]) {
			os << mutation_desc((MUTATION)i) << '=' << _mutation[(MUTATION)i] << endl;
		}
	}
}

CReproduction::RANDOM_COMPONENT CReproduction::crossover_random() const { // should there be something? prob controlled by _crossover_random[]
	const NUM x = CRand::frand(_crossover_random_sum);
	NUM y = 0.0F;
	do {
		for(int i=0; i<RANDOM_COMPONENT_LAST; i++) {
			y += _crossover_random[(RANDOM_COMPONENT)i];
			if(y > x) // found it?
				return (RANDOM_COMPONENT)i;
		}
		cerr << "WARNING: roulette past end in CReproduction:crossover_random()" << endl;
	} while(true); 
}

bool CReproduction::survive() const { // should a member of the current generation survive? prob controlled by _survive
	return (CRand::frand(1.0F) <= _survive);  // _survive is a percenhooke
}

CReproduction::MUTATION CReproduction::mutation() const { // should there be mutation? prob controlled by _entropy
	const NUM x = CRand::frand(_mutation_sum);
	NUM y = 0.0F;
	int i;
	do {
		for(i=0; i<MUTATION_LAST; ++i) {
			y += _mutation[(MUTATION)i];
			if(y > x) { // found it?
				return (MUTATION)i;
			}
		}
		cerr << "WARNING: roulette past end in CReproduction:mutation()" << endl;
	} while(true);
}

/* generates a completely random warrior using instruction biases in genus; returns number of tries it took to generate a valid warrior */
int CReproduction::generateCompletelyRandom(CWarrior &warrior) {
	unsigned int i;
	int rlen, tries=0;
	warrior.clear_info();
	while(true) { /* breaks/continues are within loop */
		tries++; // count it
		// random length and start
		warrior.len = warrior.genus()->length()->rnd(); // a random length within bounds
		warrior.start = CRand::irand(warrior.len);
		if(!warrior.is_well_formed())
			continue;
		// generate random instructions
		warrior.species()->freq()->set_warrior(&warrior);
		for(i=0; i<warrior.len; i++) {
			warrior.species()->freq()->set_warrior_index(i);
			warrior.species()->freq()->suggest_instruction(warrior.code[i],CInstGenerator::FORWARD);
		}
		// strict viability testing?
		if(CROSSOVER_BRANCH == _crossover) {
			using namespace BRANCH;
			rlen = findGenes(warrior,gRandom);
			if(!areGenesViable(gRandom,rlen))
				continue;
		}
		// do checks
		if(loops(warrior) &&
			runtest(warrior))
			break; // if here, warrior is ok
	}
	warrior.set_parents(CWarrior::NONE,CWarrior::NONE);
	warrior.clear_info();
	return tries-1; // don't count successful time
}

void CReproduction::mutate(CWarrior &warrior,const int i) const { // apply mutation to an instruction in a gene
	//TODO: farm this out to CInstGen?
	// do mutation
	switch(mutation()) {
		case CReproduction::MUT_OPCODE:
			warrior.code[i].set_opcode((OPCODE)CRand::irand(MODIFIER_LAST));
			break;
		case CReproduction::MUT_MODIFIER:
			warrior.code[i].set_modifier((MODIFIER)CRand::irand(MODIFIER_LAST));
			break;
		case CReproduction::MUT_ADDRMODE_A:
			warrior.code[i].set_addrmode_a((ADDRMODE)CRand::irand(ADDRMODE_LAST));
			break;
		case CReproduction::MUT_OPERAND_A:
			warrior.operands()->rnd(warrior.code[i],COperand::A);
			break;
		case CReproduction::MUT_ADDRMODE_B:
			warrior.code[i].set_addrmode_b((ADDRMODE)CRand::irand(ADDRMODE_LAST));
			break;
		case CReproduction::MUT_OPERAND_B:
			warrior.operands()->rnd(warrior.code[i],COperand::B);
			break;
		case CReproduction::MUT_NONE: // drop through
		default:
			break;
	}
	// checks?
	//if(safety_checks) {
		if(!warrior.code[i].valid(warrior.kingdom()->coresize()))
			PANIC(MISC,"bad instruction",NULL);
	//}
}

/* generates a new warrior based upon two parents; despite the param naming, warriors are not sexed.
   returns the number of tries it took; warrior stub must have been created (but contents are wiped) */ 
int CReproduction::breed(CWarrior &warrior,const CWarrior &mother,const CWarrior &father) {
	switch(_crossover) {
		case CROSSOVER_RANDOM:
			return breed_random(warrior,mother,father);
			break;
		case CROSSOVER_BRANCH:
			return breed_branch(warrior,mother,father);
			break;
	}
	PANIC(MISC,"bad crossover",NULL);
}

namespace { // anon

	void copy_instruction(CWarrior &warrior,const CWarrior &parent,const int i,const int cursor,const field_t ajdustment,const field_t coresize) {
		if(cursor < parent.len) { // not off the end?
			warrior.code[i].in = parent.code[cursor].in;
			warrior.code[i].a = parent.code[cursor].a;
			if(warrior.code[i].a < i) { // need ajustment?
				warrior.code[i].a += ajdustment;
				while(coresize <= warrior.code[i].a) // fold?
					warrior.code[i].a -= coresize;
			}
			warrior.code[i].b = parent.code[cursor].b;
			if(warrior.code[i].b < i) { // need ajustment?
				warrior.code[i].b += ajdustment;
				while(coresize <= warrior.code[i].b) // fold?
					warrior.code[i].b -= coresize;
			}
			if(cursor == parent.start)
				warrior.start = i;
		} else { // off the end then
			warrior.species()->freq()->set_warrior_index(i);
			warrior.species()->freq()->suggest_instruction(warrior.code[i],CInstGenerator::FORWARD);
		}
	}
	
	void adjust_addressing_before(CWarrior &warrior,const int current,const field_t coresize) {
		for(int i=0; i<current; i++) {
			if((warrior.code[i].a >= current) && (warrior.code[i].a < warrior.len)) {
				if(coresize == ++warrior.code[i].a) // inc, possibly fold?
					warrior.code[i].a = 0; 
				
			}
			if((warrior.code[i].b >= current) && (warrior.code[i].b < warrior.len)) {
				if(coresize == ++warrior.code[i].b) // inc, possibly fold?
					warrior.code[i].b = 0; 
			}
		}
	}

} // anon namespace

int CReproduction::breed_random(CWarrior &warrior,const CWarrior &mother,const CWarrior &father) {
	unsigned int i;
	int tries=0, swap_count, swap[MAXLENGTH+2];
	bool parent_mother = true; // start with mother`
	const field_t coresize = warrior.kingdom()->coresize(); // cache it
	const CWarrior *parent = &mother;
	string info = "";
	while(true) { // breaks/continues are within loop
		tries++; // count it
		// random length and start
		warrior.len = warrior.genus()->length()->rnd(); // a random length within bounds
		warrior.start = CRand::irand(warrior.len); // a default, might be inherited from parent
		if(!warrior.is_well_formed())
			continue;
		// generate instructions
		swap_count = 0;
		warrior.species()->freq()->set_warrior(&warrior);
		for(i=0; i<warrior.len; i++)
			warrior.code[i].invalidate();
		int cursor = 0, adjustment = 0;
		bool next = false;
		RANDOM_COMPONENT rnd = crossover_random();
		for(i=0; i<warrior.len;) {
			switch(rnd) { // do crossover
				case RND_NONE:
					copy_instruction(warrior,*parent,i,cursor,adjustment,coresize);
					cursor++;
					next = true;
					break;
				case RND_CROSSOVER:
					parent_mother = !parent_mother;
					parent = (parent_mother ? &mother: &father);
					swap[swap_count++] = i;
					rnd = RND_NONE;
					break;
				case RND_INSERT:
					warrior.species()->freq()->set_warrior_index(i);
					if(i < (warrior.len-1)) { // room?
						if(_adjust_addressing) adjustment++;
						warrior.code[i].set_clear();
						copy_instruction(warrior,*parent,i+1,cursor,adjustment,coresize);
						if(_adjust_addressing) adjust_addressing_before(warrior,i,coresize);
						warrior.species()->freq()->suggest_instruction(warrior.code[i],CInstGenerator::INSERT);
						cursor++;
						i++;
					} else {
						warrior.species()->freq()->suggest_instruction(warrior.code[i],CInstGenerator::FORWARD);
					}
					next = true;
					break;
				case RND_REPLACE:
					warrior.species()->freq()->set_warrior_index(i);
					if(i < (warrior.len-1)) { // room?
						warrior.code[i].set_clear();
						copy_instruction(warrior,*parent,i+1,cursor+1,adjustment,coresize);
						warrior.species()->freq()->suggest_instruction(warrior.code[i],CInstGenerator::INSERT);
						cursor++;
						i++;
					} else {
						warrior.species()->freq()->suggest_instruction(warrior.code[i],CInstGenerator::INSERT);
					}
					cursor++;
					next = true;
					break;
				case RND_DELETE:
					cursor++;
					if(0 < adjustment)
						adjustment--;
					rnd = RND_NONE;
					break;
				default:
					PANIC(MISC,"bad RANDOM_COMPONENT",NULL);
			}
			if(next) {
				mutate(warrior,i);
				i++;
				rnd = crossover_random();
				next = false;
			}
		}
		// do checks
		if(!loops(warrior))
			continue;
		if(!runtest(warrior))
			continue;
		// if here, ok
		break;
	}
	// set info
	for(i=0; i<swap_count; ++i) {
		string_append(info,swap[i]);
		info += '\t';
	}
	warrior.set_info(info);
	// done
	return tries-1; // don't count successful time
}

int CReproduction::breed_branch(CWarrior &warrior,const CWarrior &mother,const CWarrior &father) {
	using namespace BRANCH;
	const int MAX_OUTER_LOOPS = 1000;
	bool aborted = false;
	string info = "";
	int i,j,dest, outer_tries = 0, tries = 0,
		mlen = findGenes(mother,gMother),
		flen = findGenes(father,gFather);
	if(verbose) {
		cout << endl << "---------------------------" << endl
			<< "MOTHER " << mother.uid() << " (" << mother.len << " instructions)" << endl << endl;
		dumpGenes(cout,'M',mother,gMother,mlen);
		cout << "---------------------------" << endl
			<< "FATHER " << father.uid() << " (" << father.len << " instructions)" << endl << endl;
		dumpGenes(cout,'F',father,gFather,flen);
	}
	if(!areParentsViable(mlen,flen)) {
		//TODO: should this ever happen?  hmm, to think about
		if(verbose) {
			cout << "(because parents are not viable for BRANCH breeding, a new random warrior is generated instead)" << endl; 
		}
		tries = generateCompletelyRandom(warrior);
		return tries;
	}
	if(_fitness_bias) {
		fitness_mother = mother.score();
		fitness_father = father.score();
	}
	do {
		PROGRESS_TWIDDLE
		// abandom hope?
		if(MAX_OUTER_LOOPS < ++outer_tries) {
			if(verbose) {
				cout << "(because no viable BRANCH breeding could be found after " << MAX_OUTER_LOOPS << " outer tries, a new random warrior is generated instead)" << endl;
			}
			tries += generateCompletelyRandom(warrior);
			break;
		}
		// generate
		tries += generatePlan(_dat_padding,mlen,flen,warrior.species()->length()->rnd(),_fitness_bias,aborted);
		if(aborted)
			continue;
		if(0 == plan_count)
			PANIC(MISC,"zero plan_count",NULL);
		// assemble
		dest = 0;
		warrior.len = 0;
		for(i=0; i<plan_count; ++i) { // for each 'gene'
			if(gChild[i].dat_padding) {
				if(0 != gChild[i].gene)
					PANIC(MISC,"unexpected gene set",NULL);
				for(j=0; j<gChild[i].len; ++j) {
					warrior.code[dest++].set_clear();
					warrior.len++;
				}
			} else { // proper gene
				if(0 == gChild[i].gene)
					PANIC(MISC,"expected gene unset",NULL);
				// fix start for linking phase
				gChild[i].start = dest;
				// opy gene
				for(j=gChild[i].gene->start; j<gChild[i].gene->stop; ++j) {
					// copy instruction
					switch(gChild[i].parent) {
						case GENE_PLAN::MOTHER:
							warrior.code[dest] = mother.code[j]; 
							break;
						case GENE_PLAN::FATHER:
							warrior.code[dest] = father.code[j];
							break;
						default:
							PANIC(MISC,"bad gene parent",NULL);
					}
					// mutate it
					mutate(warrior,dest);
					// and move destination pointer
					dest++;
					warrior.len++;
				}
			}
		}
		// do linking
		for(i=0; i<plan_count; ++i) {
			if(0 != gChild[i].gene) {
				if(gChild[i].gene->links) {
					if(GENE_PLAN::BAD_LINK == gChild[i].link)
						PANIC(MISC,"link expected",NULL);
					// work out where and to
					j = gChild[i].start + gChild[i].len - 1;
					dest = gChild[gChild[i].link].start;
					// and do it
					warrior.code[j].a = dest;
					warrior.code[j].set_addrmode_a(DIRECT);
				}
			}
		}
		if(MAXLENGTH < warrior.len)
			PANIC(MISC,"bad warrior length",NULL);
		// set a good start pos
		warrior.start = randomStartPos();
		// do checks
		if(!loops(warrior))
			continue;
		if(!runtest(warrior))
			continue;
		// if here, ok
		break;
	} while(true); // exit condition is inside loop
	// set info
	for(i=0; i<plan_count; ++i) {
		gChild[i].id(info);
	}
	warrior.set_info(info);
	if(verbose) {
		cout << warrior.uid() << ": " << flush << warrior.info() << endl;
	}
	// done
	return tries-1; // don't count successful time
}

const char *CReproduction::mutation_desc(const CReproduction::MUTATION mutation) {
	if((0 <= mutation) && (mutation < MUTATION_LAST))
		return MUTATION_DESCS[mutation];
	return 0; // bad
}

const char *CReproduction::random_component_desc(const CReproduction::RANDOM_COMPONENT component) {
	if((0 <= component) && (component < RANDOM_COMPONENT_LAST))
		return RANDOM_COMPONENT_DESCS[component];
	return 0; // bad
}

const char *CReproduction::crossover_desc(const CReproduction::CROSSOVER crossover) {
	if((0 <= crossover) && (crossover < CROSSOVER_LAST))
		return CROSSOVER_DESCS[crossover];
	return 0; // bad
}

CReproduction::CROSSOVER CReproduction::crossover(const char *desc) {
	int i;
	for(i=0; i<CROSSOVER_LAST; ++i) {
		if(strcasecmp(desc,CROSSOVER_DESCS[(CROSSOVER)i]) == 0)
			return (CROSSOVER)i;
	}
	PANIC(MISC,"bad crossover type",NULL);
}
